/*
 * Copyright 2010-2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  http://aws.amazon.com/apache2.0
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.amazonaws.services.chimesdkmessaging.model;

import java.io.Serializable;

/**
 * <p>
 * Stores information about a callback.
 * </p>
 */
public class ChannelMessageCallback implements Serializable {
    /**
     * <p>
     * The message ID.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 128<br/>
     * <b>Pattern: </b>[-_a-zA-Z0-9]*<br/>
     */
    private String messageId;

    /**
     * <p>
     * The message content. For Amazon Lex V2 bot responses, this field holds a
     * list of messages originating from the bot. For more information, refer to
     * <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - <br/>
     * <b>Pattern: </b>[\s\S]*<br/>
     */
    private String content;

    /**
     * <p>
     * The message metadata.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>0 - 1024<br/>
     * <b>Pattern: </b>.*<br/>
     */
    private String metadata;

    /**
     * <p>
     * The push notification configuration of the message.
     * </p>
     */
    private PushNotificationConfiguration pushNotification;

    /**
     * <p>
     * The attributes for the channel message. For Amazon Lex V2 bot responses,
     * the attributes are mapped to specific fields from the bot. For more
     * information, refer to <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     */
    private java.util.Map<String, MessageAttributeValue> messageAttributes;

    /**
     * <p>
     * The ID of the SubChannel.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 128<br/>
     * <b>Pattern: </b>[-_a-zA-Z0-9]*<br/>
     */
    private String subChannelId;

    /**
     * <p>
     * The content type of the call-back message. For Amazon Lex V2 bot
     * responses, the content type is
     * <code>application/amz-chime-lex-msgs</code> for success responses and
     * <code>application/amz-chime-lex-error</code> for failure responses. For
     * more information, refer to <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>0 - 45<br/>
     * <b>Pattern: </b>[\s\S]*<br/>
     */
    private String contentType;

    /**
     * <p>
     * The message ID.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 128<br/>
     * <b>Pattern: </b>[-_a-zA-Z0-9]*<br/>
     *
     * @return <p>
     *         The message ID.
     *         </p>
     */
    public String getMessageId() {
        return messageId;
    }

    /**
     * <p>
     * The message ID.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 128<br/>
     * <b>Pattern: </b>[-_a-zA-Z0-9]*<br/>
     *
     * @param messageId <p>
     *            The message ID.
     *            </p>
     */
    public void setMessageId(String messageId) {
        this.messageId = messageId;
    }

    /**
     * <p>
     * The message ID.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 128<br/>
     * <b>Pattern: </b>[-_a-zA-Z0-9]*<br/>
     *
     * @param messageId <p>
     *            The message ID.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public ChannelMessageCallback withMessageId(String messageId) {
        this.messageId = messageId;
        return this;
    }

    /**
     * <p>
     * The message content. For Amazon Lex V2 bot responses, this field holds a
     * list of messages originating from the bot. For more information, refer to
     * <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - <br/>
     * <b>Pattern: </b>[\s\S]*<br/>
     *
     * @return <p>
     *         The message content. For Amazon Lex V2 bot responses, this field
     *         holds a list of messages originating from the bot. For more
     *         information, refer to <a href=
     *         "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     *         >Processing responses from an AppInstanceBot</a> in the <i>Amazon
     *         Chime SDK Messaging Developer Guide</i>.
     *         </p>
     */
    public String getContent() {
        return content;
    }

    /**
     * <p>
     * The message content. For Amazon Lex V2 bot responses, this field holds a
     * list of messages originating from the bot. For more information, refer to
     * <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - <br/>
     * <b>Pattern: </b>[\s\S]*<br/>
     *
     * @param content <p>
     *            The message content. For Amazon Lex V2 bot responses, this
     *            field holds a list of messages originating from the bot. For
     *            more information, refer to <a href=
     *            "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     *            >Processing responses from an AppInstanceBot</a> in the
     *            <i>Amazon Chime SDK Messaging Developer Guide</i>.
     *            </p>
     */
    public void setContent(String content) {
        this.content = content;
    }

    /**
     * <p>
     * The message content. For Amazon Lex V2 bot responses, this field holds a
     * list of messages originating from the bot. For more information, refer to
     * <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - <br/>
     * <b>Pattern: </b>[\s\S]*<br/>
     *
     * @param content <p>
     *            The message content. For Amazon Lex V2 bot responses, this
     *            field holds a list of messages originating from the bot. For
     *            more information, refer to <a href=
     *            "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     *            >Processing responses from an AppInstanceBot</a> in the
     *            <i>Amazon Chime SDK Messaging Developer Guide</i>.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public ChannelMessageCallback withContent(String content) {
        this.content = content;
        return this;
    }

    /**
     * <p>
     * The message metadata.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>0 - 1024<br/>
     * <b>Pattern: </b>.*<br/>
     *
     * @return <p>
     *         The message metadata.
     *         </p>
     */
    public String getMetadata() {
        return metadata;
    }

    /**
     * <p>
     * The message metadata.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>0 - 1024<br/>
     * <b>Pattern: </b>.*<br/>
     *
     * @param metadata <p>
     *            The message metadata.
     *            </p>
     */
    public void setMetadata(String metadata) {
        this.metadata = metadata;
    }

    /**
     * <p>
     * The message metadata.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>0 - 1024<br/>
     * <b>Pattern: </b>.*<br/>
     *
     * @param metadata <p>
     *            The message metadata.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public ChannelMessageCallback withMetadata(String metadata) {
        this.metadata = metadata;
        return this;
    }

    /**
     * <p>
     * The push notification configuration of the message.
     * </p>
     *
     * @return <p>
     *         The push notification configuration of the message.
     *         </p>
     */
    public PushNotificationConfiguration getPushNotification() {
        return pushNotification;
    }

    /**
     * <p>
     * The push notification configuration of the message.
     * </p>
     *
     * @param pushNotification <p>
     *            The push notification configuration of the message.
     *            </p>
     */
    public void setPushNotification(PushNotificationConfiguration pushNotification) {
        this.pushNotification = pushNotification;
    }

    /**
     * <p>
     * The push notification configuration of the message.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     *
     * @param pushNotification <p>
     *            The push notification configuration of the message.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public ChannelMessageCallback withPushNotification(
            PushNotificationConfiguration pushNotification) {
        this.pushNotification = pushNotification;
        return this;
    }

    /**
     * <p>
     * The attributes for the channel message. For Amazon Lex V2 bot responses,
     * the attributes are mapped to specific fields from the bot. For more
     * information, refer to <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     *
     * @return <p>
     *         The attributes for the channel message. For Amazon Lex V2 bot
     *         responses, the attributes are mapped to specific fields from the
     *         bot. For more information, refer to <a href=
     *         "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     *         >Processing responses from an AppInstanceBot</a> in the <i>Amazon
     *         Chime SDK Messaging Developer Guide</i>.
     *         </p>
     */
    public java.util.Map<String, MessageAttributeValue> getMessageAttributes() {
        return messageAttributes;
    }

    /**
     * <p>
     * The attributes for the channel message. For Amazon Lex V2 bot responses,
     * the attributes are mapped to specific fields from the bot. For more
     * information, refer to <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     *
     * @param messageAttributes <p>
     *            The attributes for the channel message. For Amazon Lex V2 bot
     *            responses, the attributes are mapped to specific fields from
     *            the bot. For more information, refer to <a href=
     *            "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     *            >Processing responses from an AppInstanceBot</a> in the
     *            <i>Amazon Chime SDK Messaging Developer Guide</i>.
     *            </p>
     */
    public void setMessageAttributes(java.util.Map<String, MessageAttributeValue> messageAttributes) {
        this.messageAttributes = messageAttributes;
    }

    /**
     * <p>
     * The attributes for the channel message. For Amazon Lex V2 bot responses,
     * the attributes are mapped to specific fields from the bot. For more
     * information, refer to <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     *
     * @param messageAttributes <p>
     *            The attributes for the channel message. For Amazon Lex V2 bot
     *            responses, the attributes are mapped to specific fields from
     *            the bot. For more information, refer to <a href=
     *            "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     *            >Processing responses from an AppInstanceBot</a> in the
     *            <i>Amazon Chime SDK Messaging Developer Guide</i>.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public ChannelMessageCallback withMessageAttributes(
            java.util.Map<String, MessageAttributeValue> messageAttributes) {
        this.messageAttributes = messageAttributes;
        return this;
    }

    /**
     * <p>
     * The attributes for the channel message. For Amazon Lex V2 bot responses,
     * the attributes are mapped to specific fields from the bot. For more
     * information, refer to <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * The method adds a new key-value pair into MessageAttributes parameter,
     * and returns a reference to this object so that method calls can be
     * chained together.
     *
     * @param key The key of the entry to be added into MessageAttributes.
     * @param value The corresponding value of the entry to be added into
     *            MessageAttributes.
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public ChannelMessageCallback addMessageAttributesEntry(String key, MessageAttributeValue value) {
        if (null == this.messageAttributes) {
            this.messageAttributes = new java.util.HashMap<String, MessageAttributeValue>();
        }
        if (this.messageAttributes.containsKey(key))
            throw new IllegalArgumentException("Duplicated keys (" + key.toString()
                    + ") are provided.");
        this.messageAttributes.put(key, value);
        return this;
    }

    /**
     * Removes all the entries added into MessageAttributes.
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     */
    public ChannelMessageCallback clearMessageAttributesEntries() {
        this.messageAttributes = null;
        return this;
    }

    /**
     * <p>
     * The ID of the SubChannel.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 128<br/>
     * <b>Pattern: </b>[-_a-zA-Z0-9]*<br/>
     *
     * @return <p>
     *         The ID of the SubChannel.
     *         </p>
     */
    public String getSubChannelId() {
        return subChannelId;
    }

    /**
     * <p>
     * The ID of the SubChannel.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 128<br/>
     * <b>Pattern: </b>[-_a-zA-Z0-9]*<br/>
     *
     * @param subChannelId <p>
     *            The ID of the SubChannel.
     *            </p>
     */
    public void setSubChannelId(String subChannelId) {
        this.subChannelId = subChannelId;
    }

    /**
     * <p>
     * The ID of the SubChannel.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 128<br/>
     * <b>Pattern: </b>[-_a-zA-Z0-9]*<br/>
     *
     * @param subChannelId <p>
     *            The ID of the SubChannel.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public ChannelMessageCallback withSubChannelId(String subChannelId) {
        this.subChannelId = subChannelId;
        return this;
    }

    /**
     * <p>
     * The content type of the call-back message. For Amazon Lex V2 bot
     * responses, the content type is
     * <code>application/amz-chime-lex-msgs</code> for success responses and
     * <code>application/amz-chime-lex-error</code> for failure responses. For
     * more information, refer to <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>0 - 45<br/>
     * <b>Pattern: </b>[\s\S]*<br/>
     *
     * @return <p>
     *         The content type of the call-back message. For Amazon Lex V2 bot
     *         responses, the content type is
     *         <code>application/amz-chime-lex-msgs</code> for success responses
     *         and <code>application/amz-chime-lex-error</code> for failure
     *         responses. For more information, refer to <a href=
     *         "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     *         >Processing responses from an AppInstanceBot</a> in the <i>Amazon
     *         Chime SDK Messaging Developer Guide</i>.
     *         </p>
     */
    public String getContentType() {
        return contentType;
    }

    /**
     * <p>
     * The content type of the call-back message. For Amazon Lex V2 bot
     * responses, the content type is
     * <code>application/amz-chime-lex-msgs</code> for success responses and
     * <code>application/amz-chime-lex-error</code> for failure responses. For
     * more information, refer to <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>0 - 45<br/>
     * <b>Pattern: </b>[\s\S]*<br/>
     *
     * @param contentType <p>
     *            The content type of the call-back message. For Amazon Lex V2
     *            bot responses, the content type is
     *            <code>application/amz-chime-lex-msgs</code> for success
     *            responses and <code>application/amz-chime-lex-error</code> for
     *            failure responses. For more information, refer to <a href=
     *            "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     *            >Processing responses from an AppInstanceBot</a> in the
     *            <i>Amazon Chime SDK Messaging Developer Guide</i>.
     *            </p>
     */
    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    /**
     * <p>
     * The content type of the call-back message. For Amazon Lex V2 bot
     * responses, the content type is
     * <code>application/amz-chime-lex-msgs</code> for success responses and
     * <code>application/amz-chime-lex-error</code> for failure responses. For
     * more information, refer to <a href=
     * "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     * >Processing responses from an AppInstanceBot</a> in the <i>Amazon Chime
     * SDK Messaging Developer Guide</i>.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>0 - 45<br/>
     * <b>Pattern: </b>[\s\S]*<br/>
     *
     * @param contentType <p>
     *            The content type of the call-back message. For Amazon Lex V2
     *            bot responses, the content type is
     *            <code>application/amz-chime-lex-msgs</code> for success
     *            responses and <code>application/amz-chime-lex-error</code> for
     *            failure responses. For more information, refer to <a href=
     *            "https://docs.aws.amazon.com/chime-sdk/latest/dg/appinstance-bots#process-response.html"
     *            >Processing responses from an AppInstanceBot</a> in the
     *            <i>Amazon Chime SDK Messaging Developer Guide</i>.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public ChannelMessageCallback withContentType(String contentType) {
        this.contentType = contentType;
        return this;
    }

    /**
     * Returns a string representation of this object; useful for testing and
     * debugging.
     *
     * @return A string representation of this object.
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        if (getMessageId() != null)
            sb.append("MessageId: " + getMessageId() + ",");
        if (getContent() != null)
            sb.append("Content: " + getContent() + ",");
        if (getMetadata() != null)
            sb.append("Metadata: " + getMetadata() + ",");
        if (getPushNotification() != null)
            sb.append("PushNotification: " + getPushNotification() + ",");
        if (getMessageAttributes() != null)
            sb.append("MessageAttributes: " + getMessageAttributes() + ",");
        if (getSubChannelId() != null)
            sb.append("SubChannelId: " + getSubChannelId() + ",");
        if (getContentType() != null)
            sb.append("ContentType: " + getContentType());
        sb.append("}");
        return sb.toString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int hashCode = 1;

        hashCode = prime * hashCode + ((getMessageId() == null) ? 0 : getMessageId().hashCode());
        hashCode = prime * hashCode + ((getContent() == null) ? 0 : getContent().hashCode());
        hashCode = prime * hashCode + ((getMetadata() == null) ? 0 : getMetadata().hashCode());
        hashCode = prime * hashCode
                + ((getPushNotification() == null) ? 0 : getPushNotification().hashCode());
        hashCode = prime * hashCode
                + ((getMessageAttributes() == null) ? 0 : getMessageAttributes().hashCode());
        hashCode = prime * hashCode
                + ((getSubChannelId() == null) ? 0 : getSubChannelId().hashCode());
        hashCode = prime * hashCode
                + ((getContentType() == null) ? 0 : getContentType().hashCode());
        return hashCode;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;

        if (obj instanceof ChannelMessageCallback == false)
            return false;
        ChannelMessageCallback other = (ChannelMessageCallback) obj;

        if (other.getMessageId() == null ^ this.getMessageId() == null)
            return false;
        if (other.getMessageId() != null
                && other.getMessageId().equals(this.getMessageId()) == false)
            return false;
        if (other.getContent() == null ^ this.getContent() == null)
            return false;
        if (other.getContent() != null && other.getContent().equals(this.getContent()) == false)
            return false;
        if (other.getMetadata() == null ^ this.getMetadata() == null)
            return false;
        if (other.getMetadata() != null && other.getMetadata().equals(this.getMetadata()) == false)
            return false;
        if (other.getPushNotification() == null ^ this.getPushNotification() == null)
            return false;
        if (other.getPushNotification() != null
                && other.getPushNotification().equals(this.getPushNotification()) == false)
            return false;
        if (other.getMessageAttributes() == null ^ this.getMessageAttributes() == null)
            return false;
        if (other.getMessageAttributes() != null
                && other.getMessageAttributes().equals(this.getMessageAttributes()) == false)
            return false;
        if (other.getSubChannelId() == null ^ this.getSubChannelId() == null)
            return false;
        if (other.getSubChannelId() != null
                && other.getSubChannelId().equals(this.getSubChannelId()) == false)
            return false;
        if (other.getContentType() == null ^ this.getContentType() == null)
            return false;
        if (other.getContentType() != null
                && other.getContentType().equals(this.getContentType()) == false)
            return false;
        return true;
    }
}
